"""Configuration of the timings."""


class TimingConfig:  # pylint: disable = too-few-public-methods
    """Global configuration of timing."""

    enable_cache: bool = True
    overhead: float = 0.0
